Package: G4SNVHunter
Type: Package
Title: Evaluating SNV-Induced Disruption of G-Quadruplex Structures
Version: 1.3.0
Authors@R: person(given = "Rongxin", 
                  family = "Zhang", 
                  email = "rongxinzhang@outlook.com", 
                  role = c("cre", "aut"),
                  comment = c('ORCID' = "0000-0002-1643-1185"))
Description: 
    G-quadruplexes (G4s) are unique nucleic acid secondary structures 
    predominantly found in guanine-rich regions and have been shown to be 
    involved in various biological regulatory processes. 
    G4SNVHunter is an R package designed to rapidly identify genomic sequences 
    with G4-forming propensity and to accurately screen user-provided 
    single nucleotide variants—as well as other small-scale variants such as 
    indels and MNVs—for their potential to destabilize these structures.
    This allows researchers to then screen these critical variants for deeper 
    study, digging into how they might influence biological functions—think 
    gene regulation, for instance—by impairing G4 formation propensity.
Encoding: UTF-8
LazyData: false
Depends: R (>= 4.3.0)
Imports: Biostrings, stats, GenomicRanges, IRanges, Rcpp, RcppRoll,
        data.table, Seqinfo, S4Vectors, ggplot2, cowplot, progress,
        ggseqlogo, viridis, ggpointdensity, tools,
        SummarizedExperiment, VariantAnnotation, dplyr, openxlsx,
        tidyr, magrittr, ggdensity
LinkingTo: Rcpp
RoxygenNote: 7.3.2
License: MIT + file LICENSE
Suggests: knitr, BiocStyle, rmarkdown, BiocManager,
        BSgenome.Hsapiens.UCSC.hg19, DT, rtracklayer, testthat (>=
        3.0.0), RBGL
VignetteBuilder: knitr
biocViews: Epigenetics, SNP
News: NEWS.md
URL: https://github.com/rongxinzh/G4SNVHunter
BugReports: https://github.com/rongxinzh/G4SNVHunter/issues
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/G4SNVHunter
git_branch: devel
git_last_commit: ec8c408
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
NeedsCompilation: yes
Packaged: 2025-11-04 22:37:31 UTC; biocbuild
Author: Rongxin Zhang [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-1643-1185>)
Maintainer: Rongxin Zhang <rongxinzhang@outlook.com>
