# Generated by roxygen2: do not edit by hand

export(G4HunterDetect)
export(G4HunterScore)
export(G4VarImpact)
export(SNVImpactG4)
export(checkSNV)
export(exportG4)
export(exportMutG4)
export(filterSNVImpact)
export(filterVarImpact)
export(loadSequence)
export(loadVariant)
export(plotG4Info)
export(plotImpactSeq)
export(plotImpactedG4)
export(plotSNVImpact)
export(plotVarImpact)
import(GenomicRanges)
import(IRanges)
import(Rcpp)
import(RcppRoll)
import(S4Vectors)
import(Seqinfo)
import(cowplot)
import(ggplot2)
import(ggpointdensity)
import(ggseqlogo)
import(progress)
import(viridis)
importFrom(Biostrings,BString)
importFrom(Biostrings,DNAString)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,readDNAStringSet)
importFrom(Biostrings,replaceAt)
importFrom(Biostrings,reverseComplement)
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,GRangesList)
importFrom(GenomicRanges,strand)
importFrom(IRanges,IRanges)
importFrom(IRanges,findOverlaps)
importFrom(IRanges,start)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,metadata)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,subjectHits)
importFrom(Seqinfo,seqlevels)
importFrom(SummarizedExperiment,rowRanges)
importFrom(VariantAnnotation,isDelins)
importFrom(VariantAnnotation,isIndel)
importFrom(VariantAnnotation,isSNV)
importFrom(VariantAnnotation,isSubstitution)
importFrom(VariantAnnotation,isTransition)
importFrom(VariantAnnotation,readVcf)
importFrom(cowplot,draw_label)
importFrom(cowplot,ggdraw)
importFrom(cowplot,plot_grid)
importFrom(data.table,":=")
importFrom(data.table,.SD)
importFrom(data.table,CJ)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(dplyr,mutate)
importFrom(ggdensity,geom_hdr)
importFrom(ggplot2,scale_color_viridis_c)
importFrom(ggpointdensity,geom_pointdensity)
importFrom(grid,arrow)
importFrom(grid,unit)
importFrom(magrittr,"%>%")
importFrom(openxlsx,addWorksheet)
importFrom(openxlsx,createWorkbook)
importFrom(openxlsx,saveWorkbook)
importFrom(openxlsx,writeData)
importFrom(stats,setNames)
importFrom(tidyr,unnest)
importFrom(tools,file_ext)
useDynLib(G4SNVHunter)
