#' Single Nucleotide Variant GRanges Object
#'
#' This dataset contains a \code{GRanges} object storing single nucleotide 
#' variants (SNVs).
#'
#' @docType data
#' @keywords datasets
#' @name snv_gr
#' @format A \code{GRanges} object with 50,000 SNVs.
#' @usage data(snv_gr)
NULL
