#' Single Nucleotide Polymorphisms GRanges Object
#'
#' This dataset contains a \code{GRanges} object storing single nucleotide 
#' polymorphisms (SNPs).
#'
#' @docType data
#' @keywords datasets
#' @name snp_gr
#' @format A \code{GRanges} object with 30,000 SNPs.
#' @usage data(snp_gr)
NULL
