## ----echo=FALSE, results='asis'-----------------------------------------------
cat("### Please refer to the full documentation at :👉 [this link](https://rongxinzh.github.io/G4SNVHunter/G4SNVHunter.html) 👈  :)")

