---
title: "Introduction to G4SNVHunter"
package: G4SNVHunter
output: 
  html_document:
    theme: cerulean
    toc: false
vignette: >
  %\VignetteIndexEntry{Introduction to G4SNVHunter}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}  
---

<img src="images/logo.png" width="230" border="0"/>

```{r, echo=FALSE, results='asis'}
cat("### Please refer to the full documentation at :👉 [this link](https://rongxinzh.github.io/G4SNVHunter/G4SNVHunter.html) 👈  :)")
```
